
// Project: webCDwriter - Network CD Writing
// File: Config.cpp
// Author: Jrg Haeger, 25.05.1999

#include "Config.h"
#include "Error.h"

	Config::Config()
	{
		cdrecordInfo = "cdrecord";
		fileName = getenv("CDWserverConfig");
		if (fileName == NULL)
			fileName = "/etc/CDWserver/config";
		char *configDir = strDup(fileName);
		int i;
		for (i = strlen(configDir);
			i > 0 && configDir[i - 1] != '/'; i--);
		configDir[i] = 0;
		firstError = NULL;
		mkisofsInfo = "mkisofs";
		numOfWriters = 0;
		writerInfo[0] = "unknown";

		// values
		cdrecordDevArg[0] = "auto";
		cdrecordPath = "cdrecord";
		debug = "";
		defaultFilesDir = "/etc/CDWserver/default_files";
		defaultSpeed = 4;
		eMailOnError = NULL;
		group = "root";
		imageOnTheFly = 1;
		infoFile = strCat(configDir, "greeting");
		laserOn = 1;
		logDir = "/var/log/CDWserver";
		logoURL = NULL;
		maxMBytesInSpoolDir = 1600;
		maxMBytesPerSession = 800;
		maxOpenConnections = 10;
		maxOpenSessions = 1;
		maxSpeed = 8;
		mkisofsMArg[0] = "auto";
		mkisofsPath = "mkisofs";
		MP3decoding = 1;
		mpg123Path = "mpg123";
		organization = "unknown";
		portNo = 12411;
		reservedMBytes = 10;
		spoolDir = "/var/spool/CDWserver";
		statusPortNo = 12412;
		user = "root";
		waitForCD = 10;
		waitForCDMsgFile = strCat(configDir, "waitForCD");

		readConfigFileLoop();
	}

	void Config::parseLine(
		const char *lineStr, char *str1, char *str2)
	{
		str1[0] = str2[0] = 0;

		if (lineStr[0] == '#')
			return;

		int i = 0;
		while (lineStr[i] == '\t'
			|| lineStr[i] == ' ')
			i++;

		int j = 0;
		for (; lineStr[i] != 0
			&& lineStr[i] != '\n'
			&& lineStr[i] != '\t'
			&& lineStr[i] != ' '
			&& lineStr[i] != '='; i++)
			str1[j++] = lineStr[i];
		str1[j] = 0;

		while (lineStr[i] == '\t'
			|| lineStr[i] == ' '
			|| lineStr[i] == '=')
			i++;

		j = 0;
		while (lineStr[i] != 0
			&& lineStr[i] != '\n')
			str2[j++] = lineStr[i++];

		// truncate trailing whitespace
		while (j > 0
			&& (str2[j - 1] == '\t'
				|| str2[j - 1] == ' '))
			j--;
		str2[j] = 0;
	}

	void Config::readConfigFile()
	{
		if (firstError != NULL)
		{
			delete firstError;
			firstError = NULL;
		}

		readConfigFileLoop();

		log.put("Config: port=%d, statusPort=%d, laser=%s",
			portNo,	statusPortNo, laserOn? "on": "off");

		char str[4 * 1024];
		if (firstError == NULL)
		try {
			File logDir(logDir);
			if (!logDir.exists())
			{
				sprintf(str, "logDir %s does not exist",
					this->logDir);
				throw str;
			}

			File spoolDir(spoolDir);
			if (!spoolDir.exists())
			{
				sprintf(str, "spoolDir %s does not exist",
					this->spoolDir);
				throw str;
			}

			testMkisofs();
			testCdrecord();
			testCdrecord2();
			for (int i = 0; i < numOfWriters; i++)
				testCdrecord3(i);

			if (!getImageOnTheFly())
				testImageFiles();

			for (int i = 0; i < numOfWriters; i++)
			if (access(mkisofsMArg[i], R_OK) != 0)
			{
				sprintf(str, "don't have the permissions "
					"to read %s", mkisofsMArg[i]);
				throw str;
			}
		}
		catch (const char *msg)
		{
			firstError = strDup(msg);
		}
		catch (Error *error)
		{
			firstError = strDup(error->toString());
			delete error;
		}
	}

void Config::readConfigFileLoop()
{
	FILE *inStream = fopen(fileName, "r");
	if (inStream == NULL)
	{
		char str[strlen(fileName) + 1024];
		sprintf(str, "can't open configFile %s", fileName);
		firstError = strDup(str);
		return;
	}

	int lineNo = 0;
	char lineStr[1024];
	while (fgets(lineStr, sizeof lineStr, inStream) != NULL)
	{
		lineNo++;
		char str1[1024], str2[1024];
		parseLine(lineStr, str1, str2);

		try {
			if (strlen(str1) == 0 && strlen(str2) == 0)
				continue;
			else
			if (strcasecmp(str1, "cdrecordDevArg") == 0)
			{
				numOfWriters = 0;
				int bus, id, lun;
				for (char *ptr = str2;
					ptr != NULL
					&& sscanf(ptr, "%d,%d,%d",
						&bus, &id, &lun) == 3; ptr++)
				{
					char str[1024];
					sprintf(str, "%d,%d,%d", bus, id, lun);
					cdrecordDevArg[numOfWriters]
						= strDup(str);
					numOfWriters++;
					ptr = strchr(ptr, ';');
				}

				if (numOfWriters > 0)
					;
				else
				if (strcasecmp(str2, "auto") == 0)
					cdrecordDevArg[0] = "auto";
				else
					throw "invalid value";
			}
			else
			if (strcasecmp(str1, "cdrecordPath") == 0)
				cdrecordPath = strDup(str2);
			else
			if (strcasecmp(str1, "debug") == 0)
				debug = strDup(str2);
			else
			if (strcasecmp(str1, "defaultFilesDir") == 0)
				defaultFilesDir = strDup(str2);
			else
			if (strcasecmp(str1, "defaultSpeed") == 0)
				defaultSpeed = toInteger(str2);
			else
			if (strcasecmp(str1, "eMailOnError") == 0)
				eMailOnError = strDup(str2);
			else
			if (strcasecmp(str1, "group") == 0)
				group = strDup(str2);
			else
			if (strcasecmp(str1, "imageFile") == 0)
				throw "now internal, remove line";
			else
			if (strcasecmp(str1, "imageOnTheFly") == 0)
				imageOnTheFly = toBoolean(str2);
			else
			if (strcasecmp(str1, "laser") == 0)
				laserOn = toBoolean(str2);
			else
			if (strcasecmp(str1, "logDir") == 0)
				logDir = strDup(str2);
			else
			if (strcasecmp(str1, "logoURL") == 0)
				logoURL = strDup(str2);
			else
			if (strcasecmp(str1, "maxMBytesInSpoolDir") == 0)
				maxMBytesInSpoolDir = toInteger(str2);
			else
			if (strcasecmp(str1, "maxMBytesPerSession") == 0)
				maxMBytesPerSession = toInteger(str2);
			else
			if (strcasecmp(str1, "maxOpenConnections") == 0)
				maxOpenConnections = toInteger(str2);
			else
			if (strcasecmp(str1, "maxOpenSessions") == 0)
			{
				if (strcasecmp(str2, "auto") == 0)
					maxOpenSessions = 0;
				else
					maxOpenSessions = toInteger(str2);
			}
			else
			if (strcasecmp(str1, "maxSpeed") == 0)
				maxSpeed = toInteger(str2);
			else
			if (strcasecmp(str1, "maxStoreMBs") == 0)
				throw ": use maxMBytesInSpoolDir"
					" (see CDWserver/config.eg)";
			else
			if (strcasecmp(str1, "mkisofsMArg") == 0)
				mkisofsMArg[0] = strDup(str2);
			else
			if (strcasecmp(str1, "mkisofsPath") == 0)
				mkisofsPath = strDup(str2);
			else
			if (strcasecmp(str1, "MP3decoding") == 0)
				MP3decoding = toBoolean(str2);
			else
			if (strcasecmp(str1, "mpg123Path") == 0)
				mpg123Path = strDup(str2);
			else
			if (strcasecmp(str1, "organization") == 0)
				organization = strDup(str2);
			else
			if (strcasecmp(str1, "portNo") == 0)
				portNo = toInteger(str2);
			else
			if (strcasecmp(str1, "reservedMBytes") == 0)
			{
				reservedMBytes = toInteger(str2);
				if (reservedMBytes < 10)
					reservedMBytes = 10;
			}
			else
			if (strcasecmp(str1, "speed") == 0)
				throw "use defaultSpeed and maxSpeed";
			else
			if (strcasecmp(str1, "spoolDir") == 0)
				spoolDir = strDup(str2);
			else
			if (strcasecmp(str1, "statusPortNo") == 0)
				statusPortNo = toInteger(str2);
			else
			if (strcasecmp(str1, "user") == 0)
				user = strDup(str2);
			else
			if (strcasecmp(str1, "waitForCD") == 0)
				waitForCD = toInteger(str2);
			else
				throw "unknown variable";
		}
		catch (const char *msg)
		{
			char str[strlen(fileName)
				+ strlen(lineStr) + 1024];
			sprintf(str, "error in %s: line %d: \"%s=%s\": %s",
				fileName, lineNo, str1, str2, msg);
			if (firstError == NULL)
				firstError = strDup(str);
			log.put(str);
		}
	}

	if (maxOpenSessions == 0)
	{
		// calculate number of possible parallel sessions
		int n = maxMBytesInSpoolDir;
		if (imageOnTheFly)
		{
			if (n < maxMBytesPerSession)
				maxMBytesPerSession = n;
		}
		else
		{
			if (n < 2 * maxMBytesPerSession)
				maxMBytesPerSession = n / 2;
			n -= maxMBytesPerSession;
		}
		maxOpenSessions = n / maxMBytesPerSession;
	}

	fclose(inStream);
}

	void Config::testCdrecord()
	{
		Process p(getCdrecordPath());
		p.addArg("-version");
		p.start();
		const char *lineStr = p.readLine();
		if (lineStr == NULL)
			throw "can't run cdrecord";
		if (p.getResult() != 0)
			throw "error running cdrecord -version";
		char str1[strlen(lineStr) + 1], str2[strlen(lineStr) + 1];
		if (sscanf(lineStr, "%s %s", str1, str2) != 2)
			throw "can't get version of cdrecord";
		char *str3 = new char[strlen(str1) + strlen(str2) + 2];
		sprintf(str3, "%s %s", str1, str2);
		cdrecordInfo = str3;
		log.put(lineStr);
	}

void Config::testCdrecord2()
{
	Process p(getCdrecordPath());
	p.addArg("-scanbus");
	p.start();

	char CDROMs[20][20], writers[20][20];
	int numOfCDROMs = 0;
	numOfWriters = 0;
	while (1)
	{
		const char *lineStr = p.readLine();
		if (lineStr == NULL)
			break;
		if (strstr(lineStr, "CD-ROM") == NULL
			&& strstr(lineStr, "WORM") == NULL)
			continue;

		log.put(lineStr);
		int bus, ID, LUN;
		if (sscanf(lineStr, "%d,%d,%d", &bus, &ID, &LUN) != 3)
			continue;

		sprintf(CDROMs[numOfCDROMs++], "%d,%d,%d",
			bus, ID, LUN);
		const char *str = lineStr;
		for (int i = 0; i < 3; i++)
		{
			while (*str != '\'' && *str != 0)
				str++;
			if (*str != 0)
				str++;
		}
		char model[17];
		strncpy(model, str, 16);
		model[16] = 0;

		const char *patterns[] = {
			"CD-R", "CDR", "CR", "CW", "RW",
			"LTR", "MP-7", "PX-R", "PX-W", NULL };
		if (strstr(model, "CDROM") == NULL
			&& strstr(model, "CD-ROM") == NULL)
		{
			const char **p = patterns;
			while (*p != NULL && strstr(model, *p) == NULL)
				p++;
			if (*p != NULL)
				sprintf(writers[numOfWriters++], "%d,%d,%d",
					bus, ID, LUN);
		}
	}

	if (p.getResult() != 0)
		throw "error running cdrecord -scanbus";

	log.put("CDROMs = %d, writers = %d", numOfCDROMs, numOfWriters);
	if (strcasecmp(cdrecordDevArg[0], "auto") == 0)
		if (numOfWriters == 0)
			if (numOfCDROMs == 0)
				throw "Cannot autodetect cdrecordDevArg";
			else
			{
				cdrecordDevArg[0] = strDup(CDROMs[0]);
				numOfWriters = 1;
			}
		else
			for (int i = 0; i < numOfWriters; i++)
				cdrecordDevArg[i] = strDup(writers[i]);

	for (int i = 0; i < numOfWriters; i++)
	{
		int j;
		for (j = 0; j < numOfCDROMs
			&& strcmp(CDROMs[j], cdrecordDevArg[i]) != 0; j++);

		if (j == numOfCDROMs)
			throw "invalid cdrecordDevArg";

		char str[1024];
		sprintf(str, "%s/cdimage%d.iso", spoolDir, i);
		imageFiles[i] = strDup(str);
#ifdef FreeBSD
		sprintf(str, "/dev/su%s", cdrecordDevArg);
		replace(str, ",", "-");
		replace(str, ",", "-");
#else
		sprintf(str, "/dev/scd%d", j);
#endif
		mkisofsMArg[i] = strDup(str);

		log.put("cdrecordDevArg = %s mkisofsMArg = %s",
			cdrecordDevArg[i], mkisofsMArg[i]);
	}
}

void Config::testCdrecord3(int writerNo)
{
	Process p(getCdrecordPath());
	char str[4*1024];
	sprintf(str, "dev=%s", getCdrecordDevArg(writerNo));
	p.addArg(str);
	p.addArg("-inq");
	p.start();

	int errorCode = 0;
	str[0] = 0;
	while (1)
	{
		const char *lineStr = p.readLine();
		if (lineStr == NULL)
			break;

		char a[strlen(lineStr) + 1], b[strlen(lineStr) + 1];
		if (sscanf(lineStr, "%s : '%s", a, b) == 2)
		if (strcmp(a, "Vendor_info") == 0
			|| strcmp(a, "Identifikation") == 0
			|| strcmp(a, "Revision") == 0)
		{
			strcat(str, b);
			strcat(str, " ");
		}
	}

	if (p.getResult() != 0)
	{
		sprintf(str, "error running cdrecord dev=%s -inq",
			getCdrecordDevArg(writerNo));
		throw str;
	}

	if (strlen(str) >= 2 && str[strlen(str) - 2] == '\'')
		str[strlen(str) - 2] = 0;
	writerInfo[writerNo] = strDup(str);
}

void Config::testImageFiles()
{
	for (int i = 0; i < numOfWriters; i++)
	{
		FILE *outStream = fopen(imageFiles[i], "w");
		if (outStream == NULL)
			throw new Error("Can't open imageFile %s",
				imageFiles[i]);
		else
			fclose(outStream);
	}
}

void Config::testMkisofs()
{
	Process p(getMkisofsPath());
	p.addArg("-version");
	p.start();
	const char *lineStr = p.readLine();
	if (lineStr == NULL)
		throw "can't run mkisofs";
	if (p.getResult() != 0)
		throw "error running mkisofs -version";
	char str1[strlen(lineStr) + 1], str2[strlen(lineStr) + 1];
	if (sscanf(lineStr, "%s %s", str1, str2) != 2)
		throw "can't get version of mkisofs";
	char *str3 = new char[strlen(str1) + strlen(str2) + 2];
	sprintf(str3, "%s %s", str1, str2);
	mkisofsInfo = str3;
	log.put(lineStr);
}
