
#ifndef CONFIG_H
#define CONFIG_H

// Project: webCDwriter - Network CD Writing
// File: Config.h
// Author: Jrg Haeger, 25.05.1999

#include <stdio.h>
#include <string.h>

#include "File.h"
#include "Process.h"
#include "Support.h"
#include "Log.h"

const int maxNumOfWriters = 10;

/**
 * delivery of the values of the configuration file
 */
class Config
{
	const char *cdrecordInfo;
	const char *fileName;
	const char *firstError;
	const char *mkisofsInfo;
	int numOfWriters;
	const char *writerInfo[maxNumOfWriters];

	// values
	const char *cdrecordDevArg[maxNumOfWriters];
	const char *cdrecordPath;
	const char *debug;
	const char *defaultFilesDir;
	int defaultSpeed;
	const char *eMailOnError;
	const char *group;
	const char *imageFiles[maxNumOfWriters];
	boolean imageOnTheFly;
	const char *infoFile;
	boolean laserOn;
	const char *logDir;
	const char *logoURL;
	int maxMBytesInSpoolDir;
	int maxMBytesPerSession;
	int maxOpenConnections;
	int maxOpenSessions;
	int maxSpeed;
	const char *mkisofsMArg[maxNumOfWriters];
	const char *mkisofsPath;
	boolean MP3decoding;
	const char *mpg123Path;
	const char *organization;
	unsigned short int portNo;
	int reservedMBytes;
	const char *spoolDir;
	unsigned short int statusPortNo;
	const char *user;
	int waitForCD;
	const char *waitForCDMsgFile;

public:
	Config();

	const char *getCdrecordDevArg(int writerNo)
	{
		return cdrecordDevArg[writerNo];
	}
	const char *getCdrecordInfo()
	{
		return cdrecordInfo;
	}
	const char *getCdrecordPath()
	{
		return cdrecordPath;
	}
	const char *getDebug()
	{
		return debug;
	}
	const char *getDefaultFilesDir()
	{
		return defaultFilesDir;
	}
	int getDefaultSpeed()
	{
		return defaultSpeed;
	}
	const char *getEMailOnError()
	{
		return eMailOnError;
	}
	const char *getFirstError()
	{
		return firstError;
	}
	const char *getGroup()
	{
		return group;
	}
	const char *getImageFile(int writerNo)
	{
		return imageFiles[writerNo];
	}
	boolean getImageOnTheFly()
	{
		return imageOnTheFly;
	}
	const char *getInfoFile()
	{
		return infoFile;
	}
	int getLaserOn()
	{
		return laserOn;
	}
	const char *getLogDir()
	{
		return logDir;
	}
	const char *getLogoURL()
	{
		return logoURL;
	}
	int getMaxMBytesInSpoolDir()
	{
		return maxMBytesInSpoolDir;
	}
	int getMaxMBytesPerSession()
	{
		return maxMBytesPerSession;
	}
	int getMaxOpenConnections()
	{
		return maxOpenConnections;
	}
	int getMaxOpenSessions()
	{
		return maxOpenSessions;
	}
	int getMaxSpeed()
	{
		return maxSpeed;
	}
	const char *getMkisofsInfo()
	{
		return mkisofsInfo;
	}
	const char *getMkisofsMArg(int writerNo)
	{
		return mkisofsMArg[writerNo];
	}
	const char *getMkisofsPath()
	{
		return mkisofsPath;
	}
	boolean getMP3decoding()
	{
		return MP3decoding;
	}
	const char *getMpg123Path()
	{
		return mpg123Path;
	}
	int getNumOfWriters()
	{
		return numOfWriters;
	}
	const char *getOrganization()
	{
		return organization;
	}
	unsigned short int getPortNo()
	{
		return portNo;
	}
	int getReservedMBytes()
	{
		return reservedMBytes;
	}
	const char *getSpoolDir()
	{
		return spoolDir;
	}
	unsigned short int getStatusPortNo()
	{
		return statusPortNo;
	}
	const char *getUser()
	{
		return user;
	}
	int getWaitForCD()
	{
		return waitForCD;
	}
	const char *getWaitForCDMsgFile()
	{
		return waitForCDMsgFile;
	}
	const char *getWriterInfo(int writerNo)
	{
		return writerInfo[writerNo];
	}

	static void parseLine(
		const char *lineStr, char *str1, char *str2);
	void readConfigFile();
	void readConfigFileLoop();
	void testCdrecord();
	void testCdrecord2();
	void testCdrecord3(int writerNo);
	void testImageFiles();
	void testMkisofs();
};

extern Config config;

#endif
